package com.mortals.xhx.module.pay.service.impl;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.common.utils.QrCodeUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.pay.model.PayRequstQuery;
import com.mortals.xhx.module.record.model.PrintWaitQueueEntity;
import com.mortals.xhx.module.record.model.PrintWaitQueueQuery;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;
import com.mortals.xhx.pay.ICBCPayUtil;
import org.springframework.beans.BeanUtils;

import java.io.File;
import java.util.Date;
import java.util.function.Function;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.pay.dao.PayRequstDao;
import com.mortals.xhx.module.pay.model.PayRequstEntity;
import com.mortals.xhx.module.pay.service.PayRequstService;
import lombok.extern.slf4j.Slf4j;
/**
* PayRequstService
* 支付请求 service实现
*
* @author zxfei
* @date 2024-12-12
*/
@Service("payRequstService")
@Slf4j
public class PayRequstServiceImpl extends AbstractCRUDServiceImpl<PayRequstDao, PayRequstEntity, Long> implements PayRequstService {

    @Autowired
    private PrintWaitQueueService printWaitQueueService;

    @Value("${upload.path}")
    private String filePath;

    @Override
    public PayRequstEntity generatePayQrcode(PrintWaitQueueEntity printWaitQueueEntity,String ip) {
        PayRequstEntity temp = this.selectOne(new PayRequstQuery().printQueueId(printWaitQueueEntity.getId()).orderNum(printWaitQueueEntity.getOrderNum()));
        if(temp!=null){
            Long payExpire = System.currentTimeMillis() - temp.getOrderDate().getTime();
            if(payExpire < temp.getPayExpire()*1000L && temp.getPayStatus()==0){
                return temp;
            }
        }
        Date orderDate = new Date();
        String orderNo = DateUtils.getDateTime(new Date(), "yyyyMMddHHmmss") + RandomUtil.randomString("0123456789", 4);
        String qrcode = ICBCPayUtil.QrcodeGenerate(orderNo, printWaitQueueEntity.getAmount(), orderDate, ip);
        if (qrcode != null) {
            String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
            String filePath = rootPath + "file/qrcode/pay/";
            String qrcodeName = orderNo + ".png";
            File pathDir = new File(filePath);
            if (!pathDir.exists()) {
                pathDir.mkdirs();
            }
            String imagepath = filePath + qrcodeName;
            QrCodeUtil.generateQrCodeFile(qrcode, imagepath);
            PayRequstEntity payRequstEntity = new PayRequstEntity();
            payRequstEntity.initAttrValue();
            payRequstEntity.setAmount(printWaitQueueEntity.getAmount());
            payRequstEntity.setApplyId(printWaitQueueEntity.getApplyId());
            payRequstEntity.setOrderDate(orderDate);
            payRequstEntity.setPayExpire(120);
            payRequstEntity.setPrintQueueId(printWaitQueueEntity.getId());
            payRequstEntity.setOrderNum(orderNo);
            payRequstEntity.setQrcodeUrl("/file/qrcode/pay/" + qrcodeName);
            payRequstEntity.setCreateTime(orderDate);
            payRequstEntity.setCreateUserId(1L);
            this.dao.insert(payRequstEntity);
            PrintWaitQueueEntity updateP = new PrintWaitQueueEntity();
            updateP.setId(printWaitQueueEntity.getId());
            updateP.setOrderNum(orderNo);
            printWaitQueueService.update(updateP);
            return payRequstEntity;
        }

        return null;
    }

    @Override
    public String qrcodePaystatus(PrintWaitQueueEntity printWaitQueueEntity) {
        PayRequstEntity payRequstEntity = this.selectOne(new PayRequstQuery().printQueueId(printWaitQueueEntity.getId()).orderNum(printWaitQueueEntity.getOrderNum()));
        if(payRequstEntity!=null){
            String payStatus = ICBCPayUtil.qrcodePaystatus(payRequstEntity.getOrderNum(),payRequstEntity.getOrderDate());
            //交易结果标志：0：支付中请稍后查询，1：支付成功，2：支付失败，3：已撤销，4：撤销中请稍后查询，5：已全额退款，6：已部分退款，7：退款中请稍后查询
            if(StringUtils.isNotEmpty(payStatus)){
                payRequstEntity.setPayStatus(Integer.valueOf(payStatus));
                payRequstEntity.setUpdateTime(new Date());
                this.dao.update(payRequstEntity);
                printWaitQueueEntity.setPaystatus(Integer.valueOf(payStatus));
                printWaitQueueService.update(printWaitQueueEntity);
            }
            return payStatus;
        }
        return null;
    }

    public static void main(String[] args) {
        String orderNo = DateUtils.getDateTime(new Date(),"yyyyMMddHHmmss") + "_"+RandomUtil.randomString("0123456789",4);
        System.out.println(orderNo);
    }
}